/*
 * @file mx6ull_solar_l8.c
 * @brief mx6ull solar l8 board initialize.
 * @author m.hirobe@co-nss.co.jp
 * @date 2019-08-29
 */

#include <asm/arch/clock.h>
#include <asm/arch/iomux.h>
#include <asm/arch/imx-regs.h>
#include <asm/arch/crm_regs.h>
#include <asm/arch/mx6-pins.h>
#include <asm/arch/sys_proto.h>
#include <asm/gpio.h>
#include <asm/imx-common/iomux-v3.h>
#include <asm/imx-common/boot_mode.h>
#include <asm/imx-common/mxc_i2c.h>
#include <asm/io.h>
#include <common.h>
#include <i2c.h>
#include <miiphy.h>
#include <fsl_esdhc.h>
#include <linux/sizes.h>
#include <mmc.h>
#include <net.h>
#include <configs/mx6ull_solar_l8.h>
#include <mx6ull_solar_l8/eeprom.h>

DECLARE_GLOBAL_DATA_PTR;

#define UART_PAD_CTRL  (PAD_CTL_PKE | PAD_CTL_PUE |		\
	PAD_CTL_PUS_100K_UP | PAD_CTL_SPEED_MED |		\
	PAD_CTL_DSE_40ohm   | PAD_CTL_SRE_FAST  | PAD_CTL_HYS)

#define I2C_PAD_CTRL    (PAD_CTL_PKE | PAD_CTL_PUE |            \
	PAD_CTL_PUS_100K_UP | PAD_CTL_SPEED_MED |               \
	PAD_CTL_DSE_40ohm | PAD_CTL_HYS |			\
	PAD_CTL_ODE)

#define LCD_PAD_CTRL    (PAD_CTL_HYS | PAD_CTL_SPEED_MED | PAD_CTL_DSE_40ohm)

#define GPMI_PAD_CTRL0 (PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP)
#define GPMI_PAD_CTRL1 (PAD_CTL_DSE_40ohm | PAD_CTL_SPEED_MED | \
			PAD_CTL_SRE_FAST)
#define GPMI_PAD_CTRL2 (GPMI_PAD_CTRL0 | GPMI_PAD_CTRL1)

#ifdef CONFIG_SYS_I2C
#define PC MUX_PAD_CTRL(I2C_PAD_CTRL)
/* I2C2 for EEPROM */
static struct i2c_pads_info i2c_pad_info = {
	.scl = {
		.i2c_mode = MX6_PAD_UART5_TX_DATA__I2C2_SCL | PC,
	},
	.sda = {
		.i2c_mode = MX6_PAD_UART5_RX_DATA__I2C2_SDA | PC,
	},
};
#endif
int dram_init(void)
{
	gd->ram_size = imx_ddr_size();

	return 0;
}

static iomux_v3_cfg_t const uart1_pads[] = {
	MX6_PAD_UART1_TX_DATA__UART1_DCE_TX | MUX_PAD_CTRL(UART_PAD_CTRL),
	MX6_PAD_UART1_RX_DATA__UART1_DCE_RX | MUX_PAD_CTRL(UART_PAD_CTRL),
};

static void setup_iomux_uart(void)
{
	imx_iomux_v3_setup_multiple_pads(uart1_pads, ARRAY_SIZE(uart1_pads));
}

#ifdef CONFIG_NAND_MXS
static iomux_v3_cfg_t const nand_pads[] = {
	MX6_PAD_NAND_DATA00__RAWNAND_DATA00 | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
	MX6_PAD_NAND_DATA01__RAWNAND_DATA01 | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
	MX6_PAD_NAND_DATA02__RAWNAND_DATA02 | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
	MX6_PAD_NAND_DATA03__RAWNAND_DATA03 | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
	MX6_PAD_NAND_DATA04__RAWNAND_DATA04 | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
	MX6_PAD_NAND_DATA05__RAWNAND_DATA05 | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
	MX6_PAD_NAND_DATA06__RAWNAND_DATA06 | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
	MX6_PAD_NAND_DATA07__RAWNAND_DATA07 | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
	MX6_PAD_NAND_CLE__RAWNAND_CLE | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
	MX6_PAD_NAND_ALE__RAWNAND_ALE | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
	MX6_PAD_NAND_CE0_B__RAWNAND_CE0_B | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
	MX6_PAD_NAND_CE1_B__RAWNAND_CE1_B | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
	MX6_PAD_NAND_RE_B__RAWNAND_RE_B | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
	MX6_PAD_NAND_WE_B__RAWNAND_WE_B | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
	MX6_PAD_NAND_WP_B__RAWNAND_WP_B | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
	MX6_PAD_NAND_READY_B__RAWNAND_READY_B | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
	MX6_PAD_NAND_DQS__RAWNAND_DQS | MUX_PAD_CTRL(GPMI_PAD_CTRL2),
};

static void setup_gpmi_nand(void)
{
	struct mxc_ccm_reg *mxc_ccm = (struct mxc_ccm_reg *)CCM_BASE_ADDR;

	/* config gpmi nand iomux */
	imx_iomux_v3_setup_multiple_pads(nand_pads, ARRAY_SIZE(nand_pads));

	setup_gpmi_io_clk((3 << MXC_CCM_CSCDR1_BCH_PODF_OFFSET) |
			  (3 << MXC_CCM_CSCDR1_GPMI_PODF_OFFSET));

	/* enable apbh clock gating */
	setbits_le32(&mxc_ccm->CCGR0, MXC_CCM_CCGR0_APBHDMA_MASK);
}
#endif

#ifdef CONFIG_FEC_MXC
static int setup_fec(int fec_id)
{
	struct iomuxc *const iomuxc_regs = (struct iomuxc *)IOMUXC_BASE_ADDR;
	int ret;

	if (fec_id == 0) {
		if (check_module_fused(MX6_MODULE_ENET1))
			return -1;

		/*
		 * Use 50M anatop loopback REF_CLK1 for ENET1,
		 * clear gpr1[13], set gpr1[17].
		 */
		clrsetbits_le32(&iomuxc_regs->gpr[1], IOMUX_GPR1_FEC1_MASK,
				IOMUX_GPR1_FEC1_CLOCK_MUX1_SEL_MASK);
	} else {
		if (check_module_fused(MX6_MODULE_ENET2))
			return -1;

		/*
		 * Use 50M anatop loopback REF_CLK2 for ENET2,
		 * clear gpr1[14], set gpr1[18].
		 */
		clrsetbits_le32(&iomuxc_regs->gpr[1], IOMUX_GPR1_FEC2_MASK,
				IOMUX_GPR1_FEC2_CLOCK_MUX1_SEL_MASK);
	}

	ret = enable_fec_anatop_clock(fec_id, ENET_50MHZ);
	if (ret)
		return ret;

	enable_enet_clk(1);

	return 0;
}
#define PHY_SMSC_CTLSTS		(17)
#define CTLSTS_EDPWRDOWN	(1 << 13)
int board_phy_config(struct phy_device *phydev)
{
	if (phydev->drv->config)
		phydev->drv->config(phydev);

	return 0;
}
#endif

int board_mmc_get_env_dev(int devno)
{
	return devno;
}

int mmc_map_to_kernel_blk(int devno)
{
	return devno;
}

static iomux_v3_cfg_t const lcd_pads[] = {
	MX6_PAD_LCD_CLK__LCDIF_CLK | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_ENABLE__LCDIF_ENABLE | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_HSYNC__LCDIF_HSYNC | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_VSYNC__LCDIF_VSYNC | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA00__LCDIF_DATA00 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA01__LCDIF_DATA01 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA02__LCDIF_DATA02 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA03__LCDIF_DATA03 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA04__LCDIF_DATA04 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA05__LCDIF_DATA05 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA06__LCDIF_DATA06 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA07__LCDIF_DATA07 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA08__LCDIF_DATA08 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA09__LCDIF_DATA09 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA10__LCDIF_DATA10 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA11__LCDIF_DATA11 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA12__LCDIF_DATA12 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA13__LCDIF_DATA13 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA14__LCDIF_DATA14 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA15__LCDIF_DATA15 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA16__LCDIF_DATA16 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA17__LCDIF_DATA17 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA18__LCDIF_DATA18 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA19__LCDIF_DATA19 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA20__LCDIF_DATA20 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA21__LCDIF_DATA21 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA22__LCDIF_DATA22 | MUX_PAD_CTRL(LCD_PAD_CTRL),
	MX6_PAD_LCD_DATA23__LCDIF_DATA23 | MUX_PAD_CTRL(LCD_PAD_CTRL),

	/* LCD_STANDBY */
	MX6_PAD_UART1_RTS_B__GPIO1_IO19 | MUX_PAD_CTRL(NO_PAD_CTRL),

	/* backlight */
	MX6_PAD_GPIO1_IO09__GPIO1_IO09 | MUX_PAD_CTRL(NO_PAD_CTRL),
	MX6_PAD_GPIO1_IO08__GPIO1_IO08 | MUX_PAD_CTRL(NO_PAD_CTRL),
};

enum {
	LEVEL_LOW = 0,
	LEVEL_HIGH,
	LEVEL_UNDEF,
};

struct gpio_table {
	unsigned gpio;
	int level;
	char name[64];
};

static struct gpio_table gpio_init_output[] = {
	{ IMX_GPIO_NR(1,0), LEVEL_LOW, "home(blue)" },	/* LED (home blue) */
	{ IMX_GPIO_NR(1,8), LEVEL_LOW, "lcd(pwm)" },	/* LCD backlight pwm */
	{ IMX_GPIO_NR(1,9), LEVEL_LOW, "lcd(backlight)" },	/* LCD backlight */
	{ IMX_GPIO_NR(1,5), LEVEL_LOW, "led(inter1)" },	/* LED (interconnect 1) */
	{ IMX_GPIO_NR(1,6), LEVEL_LOW, "led(inter2)" },	/* LED (interconnect 2)*/
	{ IMX_GPIO_NR(1,7), LEVEL_LOW, "led(inter3)" },	/* LED (interconnect 3) */
	{ IMX_GPIO_NR(1,19), LEVEL_HIGH, "lcd standby" },	/* LCD STANDBY */
	{ IMX_GPIO_NR(1,22), LEVEL_LOW, "RS485 DE" },	/* RS485 DE */
	{ IMX_GPIO_NR(1,23), LEVEL_HIGH, "RS485 nRE" },	/* RS485 nRE */
	{ IMX_GPIO_NR(1,24), LEVEL_HIGH, "EEPROM WP" },	/* EEPROM WP */
	{ IMX_GPIO_NR(2,10), LEVEL_LOW, "Ethernet nRST" },	/* Ethernet nRST */
	{ IMX_GPIO_NR(2,11), LEVEL_LOW, "led(independ1)" },	/* LED (independence 1) */
	{ IMX_GPIO_NR(2,12), LEVEL_LOW, "led(independ2)" },	/* LED (independence 2) */
	{ IMX_GPIO_NR(2,13), LEVEL_LOW, "led(independ3)" },	/* LED (independence 3) */
	{ IMX_GPIO_NR(2,14), LEVEL_LOW, "led(today)" },	/* LED (today achievement) */
	{ IMX_GPIO_NR(2,15), LEVEL_LOW, "led(yellow)" },	/* LED (home yellow)*/
	{ IMX_GPIO_NR(4,25), LEVEL_HIGH, "wlan(wake)" },	/* WLAN WAKE */
};

static struct gpio_table gpio_init_input[] = {
	{ IMX_GPIO_NR(1,25), LEVEL_UNDEF, "drive switch" },		/* drive switch sw */
	{ IMX_GPIO_NR(1,26), LEVEL_UNDEF, "home sw" },		/* home sw */
	{ IMX_GPIO_NR(1,27), LEVEL_UNDEF, "today" },		/* today achievement sw */
};

/*
 * setup_gpio_init
 * setup gpio intial value.
 */
static void setup_gpio_init(void)
{
	int i;
	unsigned cur_io;
	int cur_level;
	char *cur_name;

	/* output */
	for (i = 0; i < ARRAY_SIZE(gpio_init_output); i++) {
		cur_io = gpio_init_output[i].gpio;
		cur_level = gpio_init_output[i].level;
		cur_name = gpio_init_output[i].name;
		gpio_request(cur_io, cur_name);
		if (gpio_direction_output(cur_io, cur_level) < 0) {
			printf ("idx:%d gpio:%d level:%d set error\n", i, cur_io, cur_level);
		}
	}

	/* wlan chip_en and reset need interval */
	gpio_request(IMX_GPIO_NR(4,27), "wlan(chip_en)");
	if (gpio_direction_output(IMX_GPIO_NR(4,27), LEVEL_HIGH) < 0) {
		printf ("wlan(chip_en) level:%d set error\n", LEVEL_HIGH);
	}
	udelay(5 * 1000);
	gpio_request(IMX_GPIO_NR(4,28), "wlan(reset)");
	if (gpio_direction_output(IMX_GPIO_NR(4,28), LEVEL_HIGH) < 0) {
		printf ("wlan(reset) level:%d set error\n", LEVEL_HIGH);
	}

	for (i = 0; i< ARRAY_SIZE(gpio_init_input); i++) {
		cur_io = gpio_init_input[i].gpio;
		cur_name = gpio_init_input[i].name;
		gpio_request(cur_io, cur_name);
		if (gpio_direction_input(cur_io) < 0) {
			printf ("idx:%d gpio:%d input set error\n", i, cur_io);
		}
	}
}

int board_early_init_f(void)
{
	setup_iomux_uart();

	return 0;
}

int board_init(void)
{
	/* Address of boot parameters */
	gd->bd->bi_boot_params = PHYS_SDRAM + 0x100;

#ifdef CONFIG_SYS_I2C
	setup_i2c(0, CONFIG_SYS_I2C_SPEED, 0x7f, &i2c_pad_info);
#endif

#ifdef	CONFIG_FEC_MXC
	setup_fec(CONFIG_FEC_ENET_DEV);
#endif

#ifdef CONFIG_NAND_MXS
	setup_gpmi_nand();
#endif

    /* set iomuxc for lcdif */
	imx_iomux_v3_setup_multiple_pads(lcd_pads, ARRAY_SIZE(lcd_pads));


	setup_gpio_init();

	return 0;
}
#ifdef CONFIG_CMD_BMODE
static const struct boot_mode board_boot_modes[] = {
	/* 4 bit bus width */
	{"sd1", MAKE_CFGVAL(0x42, 0x20, 0x00, 0x00)},
	{"sd2", MAKE_CFGVAL(0x40, 0x28, 0x00, 0x00)},
	{"qspi1", MAKE_CFGVAL(0x10, 0x00, 0x00, 0x00)},
	{NULL,	 0},
};
#endif
int board_late_init(void)
{
	char *ethaddr;
#ifdef CONFIG_CMD_BMODE
	add_board_boot_modes(board_boot_modes);
#endif
	setenv("tee", "no");

#ifdef CONFIG_ENV_VARS_UBOOT_RUNTIME_CONFIG
	setenv("board_name", "L8");
#endif

#ifdef CONFIG_ENV_IS_IN_MMC
	board_late_mmc_env_init();
#endif

	fw_eeprom_init();
	ethaddr = fw_eeprom_macaddr_read();
	if (ethaddr) {
		eth_setenv_enetaddr("ethaddr", (const uchar *)ethaddr);
	}
	else {
		if (getenv("ethaddr") == NULL) {
			setenv("ethaddr", __stringify(CONFIG_ETHADDR));
		}
	}

	set_wdog_reset((struct wdog_regs *)WDOG1_BASE_ADDR);

	return 0;
}

/*
 * @fn is_input_level_continue
 * @brief check gpio input level continue until usecs [us] at 100us intervals.
 * @param gpio : check gpio specify.
 * @param cont_level : expected continuous input level.
 * @param usec : continuous time [us].
 * @return  true : input level continued until usecs.
 *          false : input level changed until usecs.
 */
#define CHECK_INTERVAL_US	(100)
static bool is_input_level_continue(unsigned int gpio, unsigned int cont_level, unsigned int usec)
{
	int i;

	for (i = 0; i < (usec/CHECK_INTERVAL_US); i++) {
		if (gpio_get_value(gpio) != cont_level) {
			return false;
		}
		udelay(CHECK_INTERVAL_US);
	}

	return true;
}

#define DRIVE_KEY		(IMX_GPIO_NR(1,25))  /* 運転切替 */
#define HOME_KEY		(IMX_GPIO_NR(1,26))  /* ホーム */
#define TODAY_KEY		(IMX_GPIO_NR(1,27))  /* 今日の実績 */
#define MODECHECK_US	(5 * 1000 * 1000)

#define LED_HOME_YELLOW		(IMX_GPIO_NR(2,15))
#define LED_TODAY_ACHIEVE	(IMX_GPIO_NR(2,14))
#define LED_INDEPENDENCE_1	(IMX_GPIO_NR(2,11))
#define LED_INDEPENDENCE_2	(IMX_GPIO_NR(2,12))
#define LED_INDEPENDENCE_3	(IMX_GPIO_NR(2,13))
#define LED_HOME_BLUE		(IMX_GPIO_NR(1,0))
#define LED_INTERCONNECT_1	(IMX_GPIO_NR(1,5))
#define LED_INTERCONNECT_2	(IMX_GPIO_NR(1,6))
#define LED_INTERCONNECT_3	(IMX_GPIO_NR(1,7))
/*
 * @fn bootmode_set
 * @brief check gpio sw and boot mode env setting.
 * @param none
 * @return 0 only (for register boot_init_r)
 */
int bootmode_set(void)
{
	char bootmode_str[] = "normal";
	int bootmode;

	if (gpio_get_value(TODAY_KEY) == LEVEL_LOW) {
		if (is_input_level_continue(TODAY_KEY, LEVEL_LOW, MODECHECK_US)) {
			bootmode = SOLAR_BOOTMODE_JET;
		}
		else {
			bootmode = SOLAR_BOOTMODE_NORMAL;
		}
	}
	else if (gpio_get_value(DRIVE_KEY) == LEVEL_LOW) {
		if (is_input_level_continue(DRIVE_KEY, LEVEL_LOW, MODECHECK_US)) {
			bootmode = SOLAR_BOOTMODE_TEST;
		}
		else {
			bootmode = SOLAR_BOOTMODE_NORMAL;
		}
	}
	else {
		bootmode = SOLAR_BOOTMODE_NORMAL;
	}

	switch (bootmode) {
		case SOLAR_BOOTMODE_TEST:
			strcpy(bootmode_str, "test");

			/* LED on */
			gpio_set_value(LED_HOME_YELLOW, LEVEL_HIGH);
			gpio_set_value(LED_TODAY_ACHIEVE, LEVEL_HIGH);
			gpio_set_value(LED_INDEPENDENCE_1, LEVEL_HIGH);
			gpio_set_value(LED_INDEPENDENCE_2, LEVEL_HIGH);
			gpio_set_value(LED_INDEPENDENCE_3, LEVEL_HIGH);
			gpio_set_value(LED_HOME_BLUE, LEVEL_HIGH);
			gpio_set_value(LED_INTERCONNECT_1, LEVEL_HIGH);
			gpio_set_value(LED_INTERCONNECT_2, LEVEL_HIGH);
			gpio_set_value(LED_INTERCONNECT_3, LEVEL_HIGH);
			break;
		case SOLAR_BOOTMODE_JET:
			strcpy(bootmode_str, "jet");
			break;
		case SOLAR_BOOTMODE_NORMAL:
		default:
			break;
	}

	setenv("bootmode_args", bootmode_str);
	printf ("select boot mode : %s\n", bootmode_str);
	
	return 0;
}

int checkboard(void)
{
	puts("Board: MX6ULL Solar L8\n");

	return 0;
}
